@@ -1,4 +1,3 @@
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -66,11 +65,9 @@ extern int gdSinT[];
 
 static void gdImageBrushApply (gdImagePtr im, int x, int y);
 static void gdImageTileApply (gdImagePtr im, int x, int y);
-static void gdImageAntiAliasedApply (gdImagePtr im, int x, int y);
 int gdImageGetTrueColorPixel (gdImagePtr im, int x, int y);
 
-gdImagePtr
-gdImageCreate (int sx, int sy)
+BGD_DECLARE(gdImagePtr) gdImageCreate (int sx, int sy)
 {
   int i;
   gdImagePtr im;
@@ -78,8 +75,6 @@ gdImageCreate (int sx, int sy)
   memset (im, 0, sizeof (gdImage));
   /* Row-major ever since gd 1.3 */
   im->pixels = (unsigned char **) gdMalloc (sizeof (unsigned char *) * sy);
-  im->AA_opacity =
-    (unsigned char **) gdMalloc (sizeof (unsigned char *) * sy);
   im->polyInts = 0;
   im->polyAllocated = 0;
   im->brush = 0;
@@ -89,8 +84,6 @@ gdImageCreate (int sx, int sy)
     {
       /* Row-major ever since gd 1.3 */
       im->pixels[i] = (unsigned char *) gdCalloc (sx, sizeof (unsigned char));
-      im->AA_opacity[i] =
-	(unsigned char *) gdCalloc (sx, sizeof (unsigned char));
     }
   im->sx = sx;
   im->sy = sy;
@@ -99,7 +92,6 @@ gdImageCreate (int sx, int sy)
   im->interlace = 0;
   im->thick = 1;
   im->AA = 0;
-  im->AA_polygon = 0;
   for (i = 0; (i < gdMaxColors); i++)
     {
       im->open[i] = 1;
@@ -116,16 +108,13 @@ gdImageCreate (int sx, int sy)
   return im;
 }
 
-gdImagePtr
-gdImageCreateTrueColor (int sx, int sy)
+BGD_DECLARE(gdImagePtr) gdImageCreateTrueColor (int sx, int sy)
 {
   int i;
   gdImagePtr im;
   im = (gdImage *) gdMalloc (sizeof (gdImage));
   memset (im, 0, sizeof (gdImage));
   im->tpixels = (int **) gdMalloc (sizeof (int *) * sy);
-  im->AA_opacity =
-    (unsigned char **) gdMalloc (sizeof (unsigned char *) * sy);
   im->polyInts = 0;
   im->polyAllocated = 0;
   im->brush = 0;
@@ -134,8 +123,6 @@ gdImageCreateTrueColor (int sx, int sy)
   for (i = 0; (i < sy); i++)
     {
       im->tpixels[i] = (int *) gdCalloc (sx, sizeof (int));
-      im->AA_opacity[i] =
-	(unsigned char *) gdCalloc (sx, sizeof (unsigned char));
     }
   im->sx = sx;
   im->sy = sy;
@@ -151,7 +138,6 @@ gdImageCreateTrueColor (int sx, int sy)
   im->alphaBlendingFlag = 1;
   im->thick = 1;
   im->AA = 0;
-  im->AA_polygon = 0;
   im->cx1 = 0;
   im->cy1 = 0;
   im->cx2 = im->sx - 1;
@@ -159,8 +145,7 @@ gdImageCreateTrueColor (int sx, int sy)
   return im;
 }
 
-void
-gdImageDestroy (gdImagePtr im)
+BGD_DECLARE(void) gdImageDestroy (gdImagePtr im)
 {
   int i;
   if (im->pixels)
@@ -179,14 +164,6 @@ gdImageDestroy (gdImagePtr im)
 	}
       gdFree (im->tpixels);
     }
-  if (im->AA_opacity)
-    {
-      for (i = 0; (i < im->sy); i++)
-	{
-	  gdFree (im->AA_opacity[i]);
-	}
-      gdFree (im->AA_opacity);
-    }
   if (im->polyInts)
     {
       gdFree (im->polyInts);
@@ -198,14 +175,12 @@ gdImageDestroy (gdImagePtr im)
   gdFree (im);
 }
 
-int
-gdImageColorClosest (gdImagePtr im, int r, int g, int b)
+BGD_DECLARE(int) gdImageColorClosest (gdImagePtr im, int r, int g, int b)
 {
   return gdImageColorClosestAlpha (im, r, g, b, gdAlphaOpaque);
 }
 
-int
-gdImageColorClosestAlpha (gdImagePtr im, int r, int g, int b, int a)
+BGD_DECLARE(int) gdImageColorClosestAlpha (gdImagePtr im, int r, int g, int b, int a)
 {
   int i;
   long rd, gd, bd, ad;
@@ -228,7 +203,7 @@ gdImageColorClosestAlpha (gdImagePtr im, int r, int g, int b, int a)
       bd = (im->blue[i] - b);
       /* gd 2.02: whoops, was - b (thanks to David Marwood) */
       /* gd 2.16: was blue rather than alpha! Geez! Thanks to 
-		Artur Jakub Jerzak */
+         Artur Jakub Jerzak */
       ad = (im->alpha[i] - a);
       dist = rd * rd + gd * gd + bd * bd + ad * ad;
       if (first || (dist < mindist))
@@ -383,8 +358,7 @@ HWB_to_RGB (HWBType HWB, RGBType * RGB)
 }
 #endif
 
-int
-gdImageColorClosestHWB (gdImagePtr im, int r, int g, int b)
+BGD_DECLARE(int) gdImageColorClosestHWB (gdImagePtr im, int r, int g, int b)
 {
   int i;
   /* long rd, gd, bd; */
@@ -413,14 +387,12 @@ gdImageColorClosestHWB (gdImagePtr im, int r, int g, int b)
   return ct;
 }
 
-int
-gdImageColorExact (gdImagePtr im, int r, int g, int b)
+BGD_DECLARE(int) gdImageColorExact (gdImagePtr im, int r, int g, int b)
 {
   return gdImageColorExactAlpha (im, r, g, b, gdAlphaOpaque);
 }
 
-int
-gdImageColorExactAlpha (gdImagePtr im, int r, int g, int b, int a)
+BGD_DECLARE(int) gdImageColorExactAlpha (gdImagePtr im, int r, int g, int b, int a)
 {
   int i;
   if (im->trueColor)
@@ -442,14 +414,12 @@ gdImageColorExactAlpha (gdImagePtr im, int r, int g, int b, int a)
   return -1;
 }
 
-int
-gdImageColorAllocate (gdImagePtr im, int r, int g, int b)
+BGD_DECLARE(int) gdImageColorAllocate (gdImagePtr im, int r, int g, int b)
 {
   return gdImageColorAllocateAlpha (im, r, g, b, gdAlphaOpaque);
 }
 
-int
-gdImageColorAllocateAlpha (gdImagePtr im, int r, int g, int b, int a)
+BGD_DECLARE(int) gdImageColorAllocateAlpha (gdImagePtr im, int r, int g, int b, int a)
 {
   int i;
   int ct = (-1);
@@ -493,14 +463,12 @@ gdImageColorAllocateAlpha (gdImagePtr im, int r, int g, int b, int a)
  * return a color index in one search over the color table.
  */
 
-int
-gdImageColorResolve (gdImagePtr im, int r, int g, int b)
+BGD_DECLARE(int) gdImageColorResolve (gdImagePtr im, int r, int g, int b)
 {
   return gdImageColorResolveAlpha (im, r, g, b, gdAlphaOpaque);
 }
 
-int
-gdImageColorResolveAlpha (gdImagePtr im, int r, int g, int b, int a)
+BGD_DECLARE(int) gdImageColorResolveAlpha (gdImagePtr im, int r, int g, int b, int a)
 {
   int c;
   int ct = -1;
@@ -558,8 +526,7 @@ gdImageColorResolveAlpha (gdImagePtr im, int r, int g, int b, int a)
   return op;			/* Return newly allocated color */
 }
 
-void
-gdImageColorDeallocate (gdImagePtr im, int color)
+BGD_DECLARE(void) gdImageColorDeallocate (gdImagePtr im, int color)
 {
   if (im->trueColor)
     {
@@ -569,8 +536,7 @@ gdImageColorDeallocate (gdImagePtr im, int color)
   im->open[color] = 1;
 }
 
-void
-gdImageColorTransparent (gdImagePtr im, int color)
+BGD_DECLARE(void) gdImageColorTransparent (gdImagePtr im, int color)
 {
   if (!im->trueColor)
     {
@@ -586,8 +552,7 @@ gdImageColorTransparent (gdImagePtr im, int color)
   im->transparent = color;
 }
 
-void
-gdImagePaletteCopy (gdImagePtr to, gdImagePtr from)
+BGD_DECLARE(void) gdImagePaletteCopy (gdImagePtr to, gdImagePtr from)
 {
   int i;
   int x, y, p;
@@ -664,18 +629,21 @@ gdImagePaletteCopy (gdImagePtr to, gdImagePtr from)
    the second call!)  The code is simplified from that in the article,
    as we know that gd images always start at (0,0) */
 
+/* 2.0.26, TBB: we now have to respect a clipping rectangle, it won't
+	necessarily start at 0. */
+
 static int
-clip_1d (int *x0, int *y0, int *x1, int *y1, int maxdim)
+clip_1d (int *x0, int *y0, int *x1, int *y1, int mindim, int maxdim)
 {
   double m;			/* gradient of line */
-  if (*x0 < 0)
+  if (*x0 < mindim)
     {				/* start of line is left of window */
-      if (*x1 < 0)		/* as is the end, so the line never cuts the window */
+      if (*x1 < mindim)		/* as is the end, so the line never cuts the window */
 	return 0;
       m = (*y1 - *y0) / (double) (*x1 - *x0);	/* calculate the slope of the line */
       /* adjust x0 to be on the left boundary (ie to be zero), and y0 to match */
-      *y0 -= m * *x0;
-      *x0 = 0;
+      *y0 -= m * (*x0 - mindim);
+      *x0 = mindim;
       /* now, perhaps, adjust the far end of the line as well */
       if (*x1 > maxdim)
 	{
@@ -694,10 +662,10 @@ clip_1d (int *x0, int *y0, int *x1, int *y1, int maxdim)
 					   boundary */
       *x0 = maxdim;
       /* now, perhaps, adjust the end of the line */
-      if (*x1 < 0)
+      if (*x1 < mindim)
 	{
-	  *y1 -= m * *x1;
-	  *x1 = 0;
+	  *y1 -= m * (*x1 - mindim);
+	  *x1 = mindim;
 	}
       return 1;
     }
@@ -709,11 +677,11 @@ clip_1d (int *x0, int *y0, int *x1, int *y1, int maxdim)
       *x1 = maxdim;
       return 1;
     }
-  if (*x1 < 0)
+  if (*x1 < mindim)
     {				/* other end is outside to the left */
       m = (*y1 - *y0) / (double) (*x1 - *x0);	/* calculate the slope of the line */
-      *y1 -= m * *x1;
-      *x1 = 0;
+      *y1 -= m * (*x1 - mindim);
+      *x1 = mindim;
       return 1;
     }
   /* only get here if both points are inside the window */
@@ -722,8 +690,7 @@ clip_1d (int *x0, int *y0, int *x1, int *y1, int maxdim)
 
 /* end of line clipping code */
 
-void
-gdImageSetPixel (gdImagePtr im, int x, int y, int color)
+BGD_DECLARE(void) gdImageSetPixel (gdImagePtr im, int x, int y, int color)
 {
   int p;
   switch (color)
@@ -764,7 +731,9 @@ gdImageSetPixel (gdImagePtr im, int x, int y, int color)
       gdImageTileApply (im, x, y);
       break;
     case gdAntiAliased:
-      gdImageAntiAliasedApply (im, x, y);
+      /* This shouldn't happen (2.0.26) because we just call
+        gdImageAALine now, but do something sane. */
+      gdImageSetPixel(im, x, y, im->AA_color);
       break;
     default:
       if (gdImageBoundsSafeMacro (im, x, y))
@@ -936,72 +905,7 @@ gdImageTileApply (gdImagePtr im, int x, int y)
     }
 }
 
-static void
-gdImageAntiAliasedApply (gdImagePtr im, int px, int py)
-{
-  float p_dist, p_alpha;
-  unsigned char opacity;
-
-  /* 
-   * Find the perpendicular distance from point C (px, py) to the line 
-   * segment AB that is being drawn.  (Adapted from an algorithm from the
-   * comp.graphics.algorithms FAQ.)
-   */
-
-  int LAC_2, LBC_2;
-
-  int Ax_Cx = im->AAL_x1 - px;
-  int Ay_Cy = im->AAL_y1 - py;
-
-  int Bx_Cx = im->AAL_x2 - px;
-  int By_Cy = im->AAL_y2 - py;
-  /* 2.0.13: bounds check! AA_opacity is just as capable of
-    overflowing as the main pixel array. Arne Jorgensen. 
-    2.0.14: typo fixed. 2.0.15: moved down below declarations
-    to satisfy non-C++ compilers. */
-  if (!gdImageBoundsSafeMacro (im, px, py))
-  {
-    return;
-  }
-  /* Get the squares of the lengths of the segemnts AC and BC. */
-  LAC_2 = (Ax_Cx * Ax_Cx) + (Ay_Cy * Ay_Cy);
-  LBC_2 = (Bx_Cx * Bx_Cx) + (By_Cy * By_Cy);
-
-  if (((im->AAL_LAB_2 + LAC_2) >= LBC_2) &&
-      ((im->AAL_LAB_2 + LBC_2) >= LAC_2))
-    {
-      /* The two angles are acute.  The point lies inside the portion of the 
-       * plane spanned by the line segment. */
-      p_dist = fabs ((float) ((Ay_Cy * im->AAL_Bx_Ax) -
-			      (Ax_Cx * im->AAL_By_Ay)) / im->AAL_LAB);
-    }
-  else
-    {
-      /* The point is past an end of the line segment.  It's length from the 
-       * segment is the shorter of the lengths from the endpoints, but call
-       * the distance -1, so as not to compute the alpha nor draw the pixel.
-       */
-      p_dist = -1;
-    }
-
-  if ((p_dist >= 0) && (p_dist <= (float) (im->thick)))
-    {
-      p_alpha = pow (1.0 - (p_dist / 1.5), 2);
-
-      if (p_alpha > 0)
-	{
-	  if (p_alpha >= 1)
-	    opacity = 255;
-	  else
-	    opacity = (unsigned char) (p_alpha * 255.0);
-	  if (!(im->AA_polygon) || (im->AA_opacity[py][px] < opacity))
-	    im->AA_opacity[py][px] = opacity;
-	}
-    }
-}
-
-int
-gdImageGetPixel (gdImagePtr im, int x, int y)
+BGD_DECLARE(int) gdImageGetPixel (gdImagePtr im, int x, int y)
 {
   if (gdImageBoundsSafeMacro (im, x, y))
     {
@@ -1036,99 +940,41 @@ gdImageGetTrueColorPixel (gdImagePtr im, int x, int y)
     }
 }
 
-void
-gdImageAABlend (gdImagePtr im)
+BGD_DECLARE(void) gdImageAABlend (gdImagePtr im)
 {
-  float p_alpha, old_alpha;
-  int color = im->AA_color, color_red, color_green, color_blue;
-  int old_color, old_red, old_green, old_blue;
-  int p_color, p_red, p_green, p_blue;
-  int px, py;
-
-  color_red = gdImageRed (im, color);
-  color_green = gdImageGreen (im, color);
-  color_blue = gdImageBlue (im, color);
-
-  /* Impose the anti-aliased drawing on the image. */
-  for (py = 0; py < im->sy; py++)
-    {
-      for (px = 0; px < im->sx; px++)
-	{
-	  if (im->AA_opacity[py][px] != 0)
-	    {
-	      old_color = gdImageGetPixel (im, px, py);
-
-	      if ((old_color != color)
-		  && ((old_color != im->AA_dont_blend)
-		      || (im->AA_opacity[py][px] == 255)))
-		{
-		  /* Only blend with different colors that aren't the 
-		   * dont_blend color. */
-		  p_alpha = (float) (im->AA_opacity[py][px]) / 255.0;
-		  old_alpha = 1.0 - p_alpha;
-
-		  if (p_alpha >= 1.0)
-		    p_color = color;
-		  else
-		    {
-		      old_red = gdImageRed (im, old_color);
-		      old_green = gdImageGreen (im, old_color);
-		      old_blue = gdImageBlue (im, old_color);
-
-		      p_red = (int) (((float) color_red * p_alpha) +
-				     ((float) old_red * old_alpha));
-		      p_green = (int) (((float) color_green * p_alpha) +
-				       ((float) old_green * old_alpha));
-		      p_blue = (int) (((float) color_blue * p_alpha) +
-				      ((float) old_blue * old_alpha));
-		      p_color =
-			gdImageColorResolve (im, p_red, p_green, p_blue);
-		    }
-		  gdImageSetPixel (im, px, py, p_color);
-		}
-	    }
-	}
-      /* Clear the AA_opacity array behind us. */
-      memset (im->AA_opacity[py], 0, im->sx);
-    }
+  /* NO-OP, kept for library compatibility. */
 }
 
+static void gdImageAALine (gdImagePtr im, int x1, int y1, int x2, int y2, int col);
+
 /* Bresenham as presented in Foley & Van Dam */
-void
-gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
+BGD_DECLARE(void) gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 {
   int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
   int wid;
   int w, wstart;
-  int thick = im->thick;
-
-  /* 2.0.10: Nick Atty: clip to edges of drawing rectangle, return if no
-     points need to be drawn */
-
-  if (clip_1d (&x1, &y1, &x2, &y2, gdImageSX (im)) == 0)
-    return;
-  if (clip_1d (&y1, &x1, &y2, &x2, gdImageSY (im)) == 0)
-    return;
-
-  /* gdAntiAliased passed as color: set anti-aliased line (AAL) global vars. */
+  int thick;
   if (color == gdAntiAliased)
     {
-      im->AAL_x1 = x1;
-      im->AAL_y1 = y1;
-      im->AAL_x2 = x2;
-      im->AAL_y2 = y2;
-
-      /* Compute what we can for point-to-line distance calculation later. */
-      im->AAL_Bx_Ax = x2 - x1;
-      im->AAL_By_Ay = y2 - y1;
-      im->AAL_LAB_2 =
-	(im->AAL_Bx_Ax * im->AAL_Bx_Ax) + (im->AAL_By_Ay * im->AAL_By_Ay);
-      im->AAL_LAB = sqrt (im->AAL_LAB_2);
-
-      /* For AA, we must draw pixels outside the width of the line.  Keep in
-       * mind that this will be curtailed by cos/sin of theta later. */
-      thick += 4;
+      /* 
+        gdAntiAliased passed as color: use the much faster, much cheaper
+        and equally attractive gdImageAALine implementation. That
+        clips too, so don't clip twice.
+      */
+      gdImageAALine(im, x1, y1, x2, y2, im->AA_color); 
+      return;
     }
+  /* 2.0.10: Nick Atty: clip to edges of drawing rectangle, return if no
+     points need to be drawn. 2.0.26, TBB: clip to edges of clipping
+     rectangle. We were getting away with this because gdImageSetPixel
+     is used for actual drawing, but this is still more efficient and opens
+     the way to skip per-pixel bounds checking in the future. */
+
+  if (clip_1d (&x1, &y1, &x2, &y2, im->cx1, im->cx2) == 0)
+    return;
+  if (clip_1d (&y1, &x1, &y2, &x2, im->cy1, im->cy2) == 0)
+    return;
+  thick = im->thick;
 
   dx = abs (x2 - x1);
   dy = abs (y2 - y1);
@@ -1300,15 +1146,11 @@ gdImageLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 	}
     }
 
-  /* If this is the only line we are drawing, go ahead and blend. */
-  if ((color == gdAntiAliased) && !(im->AA_polygon))
-    gdImageAABlend (im);
 }
 static void dashedSet (gdImagePtr im, int x, int y, int color,
 		       int *onP, int *dashStepP, int wid, int vert);
 
-void
-gdImageDashedLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
+BGD_DECLARE(void) gdImageDashedLine (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 {
   int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
   int dashStep = 0;
@@ -1491,14 +1333,12 @@ dashedSet (gdImagePtr im, int x, int y, int color,
   *onP = on;
 }
 
-int
-gdImageBoundsSafe (gdImagePtr im, int x, int y)
+BGD_DECLARE(int) gdImageBoundsSafe (gdImagePtr im, int x, int y)
 {
   return gdImageBoundsSafeMacro (im, x, y);
 }
 
-void
-gdImageChar (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
+BGD_DECLARE(void) gdImageChar (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
 {
   int cx, cy;
   int px, py;
@@ -1528,8 +1368,7 @@ gdImageChar (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
     }
 }
 
-void
-gdImageCharUp (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
+BGD_DECLARE(void) gdImageCharUp (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
 {
   int cx, cy;
   int px, py;
@@ -1559,8 +1398,7 @@ gdImageCharUp (gdImagePtr im, gdFontPtr f, int x, int y, int c, int color)
     }
 }
 
-void
-gdImageString (gdImagePtr im, gdFontPtr f,
+BGD_DECLARE(void) gdImageString (gdImagePtr im, gdFontPtr f,
 	       int x, int y, unsigned char *s, int color)
 {
   int i;
@@ -1573,8 +1411,7 @@ gdImageString (gdImagePtr im, gdFontPtr f,
     }
 }
 
-void
-gdImageStringUp (gdImagePtr im, gdFontPtr f,
+BGD_DECLARE(void) gdImageStringUp (gdImagePtr im, gdFontPtr f,
 		 int x, int y, unsigned char *s, int color)
 {
   int i;
@@ -1589,8 +1426,7 @@ gdImageStringUp (gdImagePtr im, gdFontPtr f,
 
 static int strlen16 (unsigned short *s);
 
-void
-gdImageString16 (gdImagePtr im, gdFontPtr f,
+BGD_DECLARE(void) gdImageString16 (gdImagePtr im, gdFontPtr f,
 		 int x, int y, unsigned short *s, int color)
 {
   int i;
@@ -1603,8 +1439,7 @@ gdImageString16 (gdImagePtr im, gdFontPtr f,
     }
 }
 
-void
-gdImageStringUp16 (gdImagePtr im, gdFontPtr f,
+BGD_DECLARE(void) gdImageStringUp16 (gdImagePtr im, gdFontPtr f,
 		   int x, int y, unsigned short *s, int color)
 {
   int i;
@@ -1648,15 +1483,13 @@ lsqrt (long n)
    See gd_arc_f_buggy.c for a better version that doesn't 
    seem to be bug-free yet. */
 
-void
-gdImageArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e,
+BGD_DECLARE(void) gdImageArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e,
 	    int color)
 {
   gdImageFilledArc (im, cx, cy, w, h, s, e, color, gdNoFill);
 }
 
-void
-gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e,
+BGD_DECLARE(void) gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e,
 		  int color, int style)
 {
   gdPoint pts[3];
@@ -1736,14 +1569,12 @@ gdImageFilledArc (gdImagePtr im, int cx, int cy, int w, int h, int s, int e,
     }
 }
 
-void
-gdImageFilledEllipse (gdImagePtr im, int cx, int cy, int w, int h, int color)
+BGD_DECLARE(void) gdImageFilledEllipse (gdImagePtr im, int cx, int cy, int w, int h, int color)
 {
   gdImageFilledArc (im, cx, cy, w, h, 0, 360, color, gdPie);
 }
 
-void
-gdImageFillToBorder (gdImagePtr im, int x, int y, int border, int color)
+BGD_DECLARE(void) gdImageFillToBorder (gdImagePtr im, int x, int y, int border, int color)
 {
   int lastBorder;
   /* Seek left */
@@ -1826,8 +1657,7 @@ gdImageFillToBorder (gdImagePtr im, int x, int y, int border, int color)
     }
 }
 
-void
-gdImageFill (gdImagePtr im, int x, int y, int color)
+BGD_DECLARE(void) gdImageFill (gdImagePtr im, int x, int y, int color)
 {
   int lastBorder;
   int old;
@@ -1958,8 +1788,7 @@ gdImageFill (gdImagePtr im, int x, int y, int color)
     }
 }
 
-void
-gdImageRectangle (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
+BGD_DECLARE(void) gdImageRectangle (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
 {
   int x1h = x1, x1v = x1, y1h = y1, y1v = y1, x2h = x2, x2v = x2, y2h = y2,
     y2v = y2;
@@ -1987,8 +1816,7 @@ gdImageRectangle (gdImagePtr im, int x1, int y1, int x2, int y2, int color)
   gdImageLine (im, x2v, y1v, x2v, y2v, color);
 }
 
-void
-gdImageFilledRectangle (gdImagePtr im, int x1, int y1, int x2, int y2,
+BGD_DECLARE(void) gdImageFilledRectangle (gdImagePtr im, int x1, int y1, int x2, int y2,
 			int color)
 {
   int x, y;
@@ -2013,8 +1841,7 @@ gdImageFilledRectangle (gdImagePtr im, int x1, int y1, int x2, int y2,
     }
 }
 
-void
-gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX,
+BGD_DECLARE(void) gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX,
 	     int srcY, int w, int h)
 {
   int c;
@@ -2105,8 +1932,7 @@ gdImageCopy (gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX,
 
 /* This function is a substitute for real alpha channel operations,
    so it doesn't pay attention to the alpha channel. */
-void
-gdImageCopyMerge (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
+BGD_DECLARE(void) gdImageCopyMerge (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 		  int srcX, int srcY, int w, int h, int pct)
 {
 
@@ -2156,8 +1982,7 @@ gdImageCopyMerge (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 
 /* This function is a substitute for real alpha channel operations,
    so it doesn't pay attention to the alpha channel. */
-void
-gdImageCopyMergeGray (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
+BGD_DECLARE(void) gdImageCopyMergeGray (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 		      int srcX, int srcY, int w, int h, int pct)
 {
 
@@ -2223,8 +2048,7 @@ gdImageCopyMergeGray (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
     }
 }
 
-void
-gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
+BGD_DECLARE(void) gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 		    int srcX, int srcY, int dstW, int dstH, int srcW,
 		    int srcH)
 {
@@ -2246,11 +2070,11 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
   /* Fixed by Mao Morimoto 2.0.16 */
   for (i = 0; (i < srcW); i++)
     {
-      stx[i] = dstW * (i+1) / srcW - dstW * i / srcW ;
+      stx[i] = dstW * (i + 1) / srcW - dstW * i / srcW;
     }
   for (i = 0; (i < srcH); i++)
     {
-      sty[i] = dstH * (i+1) / srcH - dstH * i / srcH ;
+      sty[i] = dstH * (i + 1) / srcH - dstH * i / srcH;
     }
   for (i = 0; (i < gdMaxColors); i++)
     {
@@ -2280,8 +2104,8 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 		      mapTo = gdImageGetTrueColorPixel (src, x, y);
 		      if (gdImageGetTransparent (src) == tmp)
 			{
-                          /* 2.0.21, TK: not tox++ */
-                          tox += stx[x - srcX];
+			  /* 2.0.21, TK: not tox++ */
+			  tox += stx[x - srcX];
 			  continue;
 			}
 		    }
@@ -2292,8 +2116,8 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 		      /* Added 7/24/95: support transparent copies */
 		      if (gdImageGetTransparent (src) == mapTo)
 			{
-                          /* 2.0.21, TK: not tox++ */
-                          tox += stx[x - srcX];
+			  /* 2.0.21, TK: not tox++ */
+			  tox += stx[x - srcX];
 			  continue;
 			}
 		    }
@@ -2371,8 +2195,7 @@ gdImageCopyResized (gdImagePtr dst, gdImagePtr src, int dstX, int dstY,
 	coordinates allow accurate rotation of 
 	objects of odd-numbered width or height. */
 
-void
-gdImageCopyRotated (gdImagePtr dst,
+BGD_DECLARE(void) gdImageCopyRotated (gdImagePtr dst,
 		    gdImagePtr src,
 		    double dstX, double dstY,
 		    int srcX, int srcY,
@@ -2449,14 +2272,14 @@ gdImageCopyRotated (gdImagePtr dst,
 #define floor2(exp) ((long) exp)
 /*#define floor2(exp) floor(exp)*/
 
-void
-gdImageCopyResampled (gdImagePtr dst,
+BGD_DECLARE(void) gdImageCopyResampled (gdImagePtr dst,
 		      gdImagePtr src,
 		      int dstX, int dstY,
 		      int srcX, int srcY,
 		      int dstW, int dstH, int srcW, int srcH)
 {
   int x, y;
+  double sy1, sy2, sx1, sx2;
   if (!dst->trueColor)
     {
       gdImageCopyResized (dst, src, dstX, dstY, srcX, srcY, dstW, dstH,
@@ -2465,19 +2288,20 @@ gdImageCopyResampled (gdImagePtr dst,
     }
   for (y = dstY; (y < dstY + dstH); y++)
     {
+      sy1 = ((double) y - (double) dstY) * (double) srcH / (double) dstH;
+      sy2 = ((double) (y + 1) - (double) dstY) * (double) srcH /
+	(double) dstH;
       for (x = dstX; (x < dstX + dstW); x++)
 	{
-	  float sy1, sy2, sx1, sx2;
-	  float sx, sy;
-	  float spixels = 0;
-	  float red = 0.0, green = 0.0, blue = 0.0, alpha = 0.0;
-	  sy1 = ((float) y - (float) dstY) * (float) srcH / (float) dstH;
-	  sy2 = ((float) (y + 1) - (float) dstY) * (float) srcH /
-	    (float) dstH;
+	  double sx, sy;
+	  double spixels = 0;
+	  double red = 0.0, green = 0.0, blue = 0.0, alpha = 0.0;
+	  sx1 = ((double) x - (double) dstX) * (double) srcW / dstW;
+	  sx2 = ((double) (x + 1) - (double) dstX) * (double) srcW / dstW;
 	  sy = sy1;
 	  do
 	    {
-	      float yportion;
+	      double yportion;
 	      if (floor2 (sy) == floor2 (sy1))
 		{
 		  yportion = 1.0 - (sy - floor2 (sy));
@@ -2495,13 +2319,11 @@ gdImageCopyResampled (gdImagePtr dst,
 		{
 		  yportion = 1.0;
 		}
-	      sx1 = ((float) x - (float) dstX) * (float) srcW / dstW;
-	      sx2 = ((float) (x + 1) - (float) dstX) * (float) srcW / dstW;
 	      sx = sx1;
 	      do
 		{
-		  float xportion;
-		  float pcontribution;
+		  double xportion;
+		  double pcontribution;
 		  int p;
 		  if (floor2 (sx) == floor2 (sx1))
 		    {
@@ -2570,8 +2392,7 @@ gdImageCopyResampled (gdImagePtr dst,
     }
 }
 
-gdImagePtr
-gdImageCreateFromXbm (FILE * fd)
+BGD_DECLARE(gdImagePtr) gdImageCreateFromXbm (FILE * fd)
 {
   gdImagePtr im;
   int bit;
@@ -2696,8 +2517,7 @@ fail:
   return 0;
 }
 
-void
-gdImagePolygon (gdImagePtr im, gdPointPtr p, int n, int c)
+BGD_DECLARE(void) gdImagePolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 {
   int i;
   int lx, ly;
@@ -2706,10 +2526,6 @@ gdImagePolygon (gdImagePtr im, gdPointPtr p, int n, int c)
       return;
     }
 
-  /* Let it be known that we are drawing a polygon so that the opacity
-   * mask doesn't get cleared after each line. */
-  if (c == gdAntiAliased)
-    im->AA_polygon = 1;
 
   lx = p->x;
   ly = p->y;
@@ -2722,15 +2538,8 @@ gdImagePolygon (gdImagePtr im, gdPointPtr p, int n, int c)
       ly = p->y;
     }
 
-  if (c == gdAntiAliased)
-    {
-      im->AA_polygon = 0;
-      gdImageAABlend (im);
-    }
 }
 
-int gdCompareInt (const void *a, const void *b);
-
 /* THANKS to Kirsten Schulz for the polygon fixes! */
 
 /* The intersection finding technique of this code could be improved  */
@@ -2738,10 +2547,14 @@ int gdCompareInt (const void *a, const void *b);
 /* That could help to adjust intersections  to produce a nice */
 /* interior_extrema. */
 
-void
-gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
+static void horizontalLine(gdImagePtr im, int minx, int maxx, int y,
+	int fill_color);
+
+BGD_DECLARE(void) gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 {
   int i;
+  int j;
+  int index;
   int y;
   int miny, maxy;
   int x1, y1;
@@ -2754,10 +2567,6 @@ gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
       return;
     }
 
-  if (c == gdAntiAliased)
-    fill_color = im->AA_color;
-  else
-    fill_color = c;
 
   if (!im->polyAllocated)
     {
@@ -2787,18 +2596,27 @@ gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 	}
     }
   /* 2.0.16: Optimization by Ilia Chipitsine -- don't waste time offscreen */
-  if (miny < 0) {
-    miny = 0;
-  }
-  if (maxy >= gdImageSY(im)) {
-    maxy = gdImageSY(im) - 1;
-  } 
+  /* 2.0.26: clipping rectangle is even better */
+  if (miny < im->cy1)
+    {
+      miny = im->cy1;
+    }
+  if (maxy > im->cy2)
+    {
+      maxy = im->cy2;
+    }
   /* Fix in 1.3: count a vertex only once */
   for (y = miny; (y <= maxy); y++)
     {
 /*1.4           int interLast = 0; */
 /*              int dirLast = 0; */
 /*              int interFirst = 1; */
+      int yshift = 0;
+      if (c == gdAntiAliased) {
+        fill_color = im->AA_color;
+      } else {
+        fill_color = c;
+      }
       ints = 0;
       for (i = 0; (i < n); i++)
 	{
@@ -2834,39 +2652,73 @@ gdImageFilledPolygon (gdImagePtr im, gdPointPtr p, int n, int c)
 	  /* Do the following math as float intermediately, and round to ensure
 	   * that Polygon and FilledPolygon for the same set of points have the
 	   * same footprint. */
+ 
 	  if ((y >= y1) && (y < y2))
 	    {
-	      im->polyInts[ints++] = (float) ((y - y1) * (x2 - x1)) /
-		(float) (y2 - y1) + 0.5 + x1;
+	      im->polyInts[ints++] = (int) ((float) ((y - y1) * (x2 - x1)) /
+		(float) (y2 - y1) + 0.5 + x1);
 	    }
 	  else if ((y == maxy) && (y > y1) && (y <= y2))
 	    {
-	      im->polyInts[ints++] = (float) ((y - y1) * (x2 - x1)) /
-		(float) (y2 - y1) + 0.5 + x1;
+	      im->polyInts[ints++] = (int) ((float) ((y - y1) * (x2 - x1)) /
+		(float) (y2 - y1) + 0.5 + x1);
 	    }
 	}
-      qsort (im->polyInts, ints, sizeof (int), gdCompareInt);
-
+      /* 
+        2.0.26: polygons pretty much always have less than 100 points,
+        and most of the time they have considerably less. For such trivial
+        cases, insertion sort is a good choice. Also a good choice for
+        future implementations that may wish to indirect through a table.
+      */
+      for (i = 1; (i < ints); i++) {
+        index = im->polyInts[i];
+        j = i;
+        while ((j > 0) && (im->polyInts[j - 1] > index)) {
+          im->polyInts[j] = im->polyInts[j - 1];
+          j--;
+        }
+        im->polyInts[j] = index;
+      }
       for (i = 0; (i < (ints)); i += 2)
 	{
-	  gdImageLine (im, im->polyInts[i], y, im->polyInts[i + 1], y,
-		       fill_color);
+          int minx = im->polyInts[i];
+          int maxx = im->polyInts[i + 1];
+          horizontalLine(im, minx, maxx, y, fill_color);
 	}
     }
-
   /* If we are drawing this AA, then redraw the border with AA lines. */
-  if (c == gdAntiAliased)
+  /* This doesn't work as well as I'd like, but it doesn't clash either. */
+  if (c == gdAntiAliased) {
     gdImagePolygon (im, p, n, c);
+  } 
 }
 
-int
-gdCompareInt (const void *a, const void *b)
+static void horizontalLine(gdImagePtr im, int minx, int maxx, int y,
+	int fill_color)
 {
-  return (*(const int *) a) - (*(const int *) b);
+  /* 2.0.27: potential corruption fixed by John Ellson */
+  if (minx < im->cx1) minx = im->cx1;
+  if (maxx < minx) maxx = minx;
+  if (maxx > im->cx2) maxx = im->cx2;
+  if (minx > maxx) minx = maxx;
+                                                                                                                                        
+  if (y < im->cy1) y = im->cy1;
+  if (y > im->cy2) y = im->cy2;
+                                                                                                                                        
+  if (im->trueColor) {
+    while (minx <= maxx) {
+      im->tpixels[y][minx++] = fill_color;
+    }
+  } else {
+    while (minx <= maxx) {
+      im->pixels[y][minx++] = fill_color;
+    }
+  }
 }
 
-void
-gdImageSetStyle (gdImagePtr im, int *style, int noOfPixels)
+inline static void gdImageSetAAPixelColor(gdImagePtr im, int x, int y, int color, int t);
+
+BGD_DECLARE(void) gdImageSetStyle (gdImagePtr im, int *style, int noOfPixels)
 {
   if (im->style)
     {
@@ -2878,14 +2730,12 @@ gdImageSetStyle (gdImagePtr im, int *style, int noOfPixels)
   im->stylePos = 0;
 }
 
-void
-gdImageSetThickness (gdImagePtr im, int thickness)
+BGD_DECLARE(void) gdImageSetThickness (gdImagePtr im, int thickness)
 {
   im->thick = thickness;
 }
 
-void
-gdImageSetBrush (gdImagePtr im, gdImagePtr brush)
+BGD_DECLARE(void) gdImageSetBrush (gdImagePtr im, gdImagePtr brush)
 {
   int i;
   im->brush = brush;
@@ -2904,8 +2754,7 @@ gdImageSetBrush (gdImagePtr im, gdImagePtr brush)
     }
 }
 
-void
-gdImageSetTile (gdImagePtr im, gdImagePtr tile)
+BGD_DECLARE(void) gdImageSetTile (gdImagePtr im, gdImagePtr tile)
 {
   int i;
   im->tile = tile;
@@ -2924,30 +2773,26 @@ gdImageSetTile (gdImagePtr im, gdImagePtr tile)
     }
 }
 
-void
-gdImageSetAntiAliased (gdImagePtr im, int c)
+BGD_DECLARE(void) gdImageSetAntiAliased (gdImagePtr im, int c)
 {
   im->AA = 1;
   im->AA_color = c;
   im->AA_dont_blend = -1;
 }
 
-void
-gdImageSetAntiAliasedDontBlend (gdImagePtr im, int c, int dont_blend)
+BGD_DECLARE(void) gdImageSetAntiAliasedDontBlend (gdImagePtr im, int c, int dont_blend)
 {
   im->AA = 1;
   im->AA_color = c;
   im->AA_dont_blend = dont_blend;
 }
 
-void
-gdImageInterlace (gdImagePtr im, int interlaceArg)
+BGD_DECLARE(void) gdImageInterlace (gdImagePtr im, int interlaceArg)
 {
   im->interlace = interlaceArg;
 }
 
-int
-gdImageCompare (gdImagePtr im1, gdImagePtr im2)
+BGD_DECLARE(int) gdImageCompare (gdImagePtr im1, gdImagePtr im2)
 {
   int x, y;
   int p1, p2;
@@ -3039,8 +2884,7 @@ gdImageCompare (gdImagePtr im1, gdImagePtr im2)
   return cmpStatus;
 }
 
-int
-gdAlphaBlend (int dst, int src)
+BGD_DECLARE(int) gdAlphaBlend (int dst, int src)
 {
   /* 2.0.12: TBB: alpha in the destination should be a 
      component of the result. Thanks to Frank Warmerdam for
@@ -3061,53 +2905,57 @@ gdAlphaBlend (int dst, int src)
 	    gdTrueColorGetBlue (dst)) / gdAlphaMax));
 }
 
-void
-gdImageAlphaBlending (gdImagePtr im, int alphaBlendingArg)
+BGD_DECLARE(void) gdImageAlphaBlending (gdImagePtr im, int alphaBlendingArg)
 {
   im->alphaBlendingFlag = alphaBlendingArg;
 }
 
-void
-gdImageSaveAlpha (gdImagePtr im, int saveAlphaArg)
+BGD_DECLARE(void) gdImageSaveAlpha (gdImagePtr im, int saveAlphaArg)
 {
   im->saveAlphaFlag = saveAlphaArg;
 }
 
-void
-gdImageSetClip (gdImagePtr im, int x1, int y1, int x2, int y2)
+BGD_DECLARE(void) gdImageSetClip (gdImagePtr im, int x1, int y1, int x2, int y2)
 {
-  if (x1 < 0) {
-    x1 = 0;
-  }
-  if (x1 >= im->sx) {
-    x1 = im->sx - 1;
-  }
-  if (x2 < 0) {
-    x2 = 0;
-  }
-  if (x2 >= im->sx) {
-    x2 = im->sx - 1;
-  }
-  if (y1 < 0) {
-    y1 = 0;
-  }
-  if (y1 >= im->sy) {
-    y1 = im->sy - 1;
-  }
-  if (y2 < 0) {
-    y2 = 0;
-  }
-  if (y2 >= im->sy) {
-    y2 = im->sy - 1;
-  }
+  if (x1 < 0)
+    {
+      x1 = 0;
+    }
+  if (x1 >= im->sx)
+    {
+      x1 = im->sx - 1;
+    }
+  if (x2 < 0)
+    {
+      x2 = 0;
+    }
+  if (x2 >= im->sx)
+    {
+      x2 = im->sx - 1;
+    }
+  if (y1 < 0)
+    {
+      y1 = 0;
+    }
+  if (y1 >= im->sy)
+    {
+      y1 = im->sy - 1;
+    }
+  if (y2 < 0)
+    {
+      y2 = 0;
+    }
+  if (y2 >= im->sy)
+    {
+      y2 = im->sy - 1;
+    }
   im->cx1 = x1;
   im->cy1 = y1;
   im->cx2 = x2;
   im->cy2 = y2;
 }
 
-void
-gdImageGetClip (gdImagePtr im, int *x1P, int *y1P, int *x2P, int *y2P)
+BGD_DECLARE(void) gdImageGetClip (gdImagePtr im, int *x1P, int *y1P, int *x2P, int *y2P)
 {
   *x1P = im->cx1;
   *y1P = im->cy1;
@@ -3115,4 +2963,102 @@ gdImageGetClip (gdImagePtr im, int *x1P, int *y1P, int *x2P, int *y2P)
   *y2P = im->cy2;
 }
 
+/*
+ * Added on 2003/12 by Pierre-Alain Joye (pajoye@pearfr.org)
+ * */
+#define BLEND_COLOR(a, nc, c, cc) \
+nc = (cc) + (((((c) - (cc)) * (a)) + ((((c) - (cc)) * (a)) >> 8) + 0x80) >> 8);
 
+inline static void gdImageSetAAPixelColor(gdImagePtr im, int x, int y, int color, int t)
+{
+	int dr,dg,db,p,r,g,b;
+	p = gdImageGetPixel(im,x,y);
+        /* TBB: we have to implement the dont_blend stuff to provide
+          the full feature set of the old implementation */
+        if ((p == color)
+	  || ((p == im->AA_dont_blend)
+	      && (t != 0x00)))
+        {
+          return;
+        }
+	dr = gdTrueColorGetRed(color);
+	dg = gdTrueColorGetGreen(color);
+	db = gdTrueColorGetBlue(color);
+
+	r = gdTrueColorGetRed(p);
+	g = gdTrueColorGetGreen(p);
+	b = gdTrueColorGetBlue(p);
+
+	BLEND_COLOR(t, dr, r, dr);
+	BLEND_COLOR(t, dg, g, dg);
+	BLEND_COLOR(t, db, b, db);
+	im->tpixels[y][x]=gdTrueColorAlpha(dr, dg, db,  gdAlphaOpaque);
+}  
+
+static void gdImageAALine (gdImagePtr im, int x1, int y1, int x2, int y2, int col)
+{
+	/* keep them as 32bits */
+	long x, y, inc;
+	long dx, dy,tmp;
+	if (!im->trueColor) {
+		/* TBB: don't crash when the image is of the wrong type */
+		gdImageLine(im, x1, y1, x2, y2, col);
+		return;
+	}
+        /* TBB: use the clipping rectangle */
+        if (clip_1d (&x1, &y1, &x2, &y2, im->cx1, im->cx2) == 0)
+          return;
+        if (clip_1d (&y1, &x1, &y2, &x2, im->cy1, im->cy2) == 0)
+          return;
+	dx = x2 - x1;
+	dy = y2 - y1;
+
+	if (dx == 0 && dy == 0) {
+		/* TBB: allow setting points */
+		gdImageSetAAPixelColor(im, x1, y1, col, 0xFF);
+		return;
+	}
+	if (abs(dx) > abs(dy)) {
+		if (dx < 0) {
+			tmp = x1;
+			x1 = x2;
+			x2 = tmp;
+			tmp = y1;
+			y1 = y2;
+			y2 = tmp;
+			dx = x2 - x1;
+			dy = y2 - y1;
+		}
+		x = x1 << 16;
+		y = y1 << 16;
+		inc = (dy * 65536) / dx;
+		/* TBB: set the last pixel for consistency (<=) */
+		while ((x >> 16) <= x2) {
+			gdImageSetAAPixelColor(im, x >> 16, y >> 16, col, (y >> 8) & 0xFF);
+			gdImageSetAAPixelColor(im, x >> 16, (y >> 16) + 1,col, (~y >> 8) & 0xFF);
+			x += (1 << 16);
+			y += inc;
+		}
+	} else {
+		if (dy < 0) {
+			tmp = x1;
+			x1 = x2;
+			x2 = tmp;
+			tmp = y1;
+			y1 = y2;
+			y2 = tmp;
+			dx = x2 - x1;
+			dy = y2 - y1;
+		}
+		x = x1 << 16;
+		y = y1 << 16;
+		inc = (dx * 65536) / dy;
+		/* TBB: set the last pixel for consistency (<=) */
+		while ((y>>16) <= y2) {
+			gdImageSetAAPixelColor(im, x >> 16, y >> 16, col, (x >> 8) & 0xFF);
+			gdImageSetAAPixelColor(im, (x >> 16) + 1, (y >> 16),col, (~x >> 8) & 0xFF);
+			x += inc;
+			y += (1<<16);
+		}
+	}
+}
