@@ -19,7 +19,8 @@ extern "C"
 
 #include "gd.h"
 
-   BGD_EXPORT  gdFontPtr gdFontMediumBold;
+   BGD_EXPORT_DATA_PROT  gdFontPtr gdFontMediumBold;
+   BGD_DECLARE(gdFontPtr) gdFontGetMediumBold(void);
 
 #ifdef __cplusplus
 }
