@@ -15,6 +15,7 @@
 #include <math.h>
 #include "gd.h"
 #include "gdhelpers.h"
+#include "entities.h"
 
 /* 2.0.10: WIN32, not MSWIN32 */
 #ifndef WIN32
@@ -27,8 +28,7 @@
 /* number of antialised colors for indexed bitmaps */
 #define NUMCOLORS 8
 
-char *
-gdImageStringTTF (gdImage * im, int *brect, int fg, char *fontlist,
+BGD_DECLARE(char *) gdImageStringTTF (gdImage * im, int *brect, int fg, char *fontlist,
 		  double ptsize, double angle, int x, int y, char *string)
 {
   /* 2.0.6: valid return */
@@ -37,16 +37,14 @@ gdImageStringTTF (gdImage * im, int *brect, int fg, char *fontlist,
 }
 
 #ifndef HAVE_LIBFREETYPE
-char *
-gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
+BGD_DECLARE(char *) gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
 		   double ptsize, double angle, int x, int y, char *string,
 		   gdFTStringExtraPtr strex)
 {
   return "libgd was not built with FreeType font support\n";
 }
 
-char *
-gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
+BGD_DECLARE(char *) gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 		 double ptsize, double angle, int x, int y, char *string)
 {
   return "libgd was not built with FreeType font support\n";
@@ -188,6 +186,12 @@ tweencolorkey_t;
 #include "jisx0208.h"
 #endif
 
+static int comp_entities(const void *e1, const void *e2) {
+  struct entities_s *en1 = (struct entities_s *) e1;
+  struct entities_s *en2 = (struct entities_s *) e2;
+  return strcmp(en1->name, en2->name);
+}
+
 #define Tcl_UniChar int
 #define TCL_UTF_MAX 3
 static int
@@ -196,8 +200,12 @@ gdTcl_UtfToUniChar (char *str, Tcl_UniChar * chPtr)
 /* chPtr is the int for the result */
 {
   int byte;
+  char entity_name_buf[ENTITY_NAME_LENGTH_MAX+1];
+  char *p;
+  struct entities_s key, *res;
 
   /* HTML4.0 entities in decimal form, e.g. &#197; */
+  /*           or in hexadecimal form, e.g. &#x6C34; */
   byte = *((unsigned char *) str);
   if (byte == '&')
     {
@@ -206,15 +214,33 @@ gdTcl_UtfToUniChar (char *str, Tcl_UniChar * chPtr)
       byte = *((unsigned char *) (str + 1));
       if (byte == '#')
 	{
-	  for (i = 2; i < 8; i++)
-	    {
-	      byte = *((unsigned char *) (str + i));
-	      if (byte >= '0' && byte <= '9')
-		{
-		  n = (n * 10) + (byte - '0');
+          byte = *((unsigned char *) (str + 2));
+          if (byte == 'x' || byte == 'X')
+            {
+              for (i = 3; i < 8; i++)
+                {
+                  byte = *((unsigned char *) (str + i));
+                  if (byte >= 'A' && byte <= 'F')
+                    byte = byte - 'A' + 10;
+                  else if (byte >= 'a' && byte <= 'f')
+                    byte = byte - 'a' + 10;
+                  else if (byte >= '0' && byte <= '9')
+                    byte = byte - '0';
+                  else
+                    break;
+                  n = (n * 16) + byte;
+                }
+            }
+          else
+            {
+	      for (i = 2; i < 8; i++)
+	        {
+	          byte = *((unsigned char *) (str + i));
+	          if (byte >= '0' && byte <= '9')
+	            n = (n * 10) + (byte - '0');
+	          else
+		    break;
 		}
-	      else
-		break;
 	    }
 	  if (byte == ';')
 	    {
@@ -222,6 +248,29 @@ gdTcl_UtfToUniChar (char *str, Tcl_UniChar * chPtr)
 	      return ++i;
 	    }
 	}
+      else
+        {
+          key.name = p = entity_name_buf;
+          for (i = 1; i < 1 + ENTITY_NAME_LENGTH_MAX; i++)
+            {
+              byte = *((unsigned char *) (str + i));
+              if (byte == '\0')
+                break;
+              if (byte == ';')
+                {
+                  *p++ = '\0';
+                  res = bsearch(&key, entities, NR_OF_ENTITIES,
+                       sizeof(entities[0]), *comp_entities);
+                  if (res)
+                    {
+                      *chPtr = (Tcl_UniChar) res->value;
+                      return ++i;
+                    }
+                  break;
+               }
+              *p++ = byte;
+            }
+        }
     }
 
   /*
@@ -386,8 +435,9 @@ fontFetch (char **error, void *key)
        * Allocate an oversized buffer that is guaranteed to be
        * big enough for all paths to be tested.
        */
+      /* 2.0.22: Thorben Kundinger: +8 is needed, not +6. */
       fullname = gdRealloc (fullname,
-			    strlen (fontsearchpath) + strlen (name) + 6);
+			    strlen (fontsearchpath) + strlen (name) + 8);
       /* if name is an absolute or relative pathname then test directly */
       if (strchr (name, '/')
 	  || (name[0] != 0 && name[1] == ':'
@@ -397,14 +447,27 @@ fontFetch (char **error, void *key)
 	  if (access (fullname, R_OK) == 0)
 	    {
 	      font_found++;
-              /* 2.0.16: memory leak fixed, Gustavo Scotti */ 		
-              gdFree(path);
+	      /* 2.0.16: memory leak fixed, Gustavo Scotti */
+	      gdFree (path);
 	      break;
 	    }
 	}
       for (dir = strtok (path, PATHSEPARATOR); dir;
 	   dir = strtok (0, PATHSEPARATOR))
 	{
+          if (strchr (name, '.'))
+            {
+              sprintf (fullname, "%s/%s", dir, name);
+              if (access (fullname, R_OK) == 0)
+                {
+                  font_found++;
+                  break;
+                }
+              else
+                {
+                  continue;
+                }
+            }
 	  sprintf (fullname, "%s/%s.ttf", dir, name);
 	  if (access (fullname, R_OK) == 0)
 	    {
@@ -479,16 +542,20 @@ fontFetch (char **error, void *key)
       platform = charmap->platform_id;
       encoding = charmap->encoding_id;
 
-/* EAM DEBUG - Newer versions of libfree2 make it easier by defining encodings */
-#if (defined(FREETYPE_MAJOR) && (FREETYPE_MAJOR >=2 ) && (FREETYPE_MINOR >= 1))
+/* EAM DEBUG - Newer versions of libfree2 make it easier 
+	by defining encodings */
+/* TBB: get this exactly right: 2.1.3 *or better*, all possible cases. */
+
+#if ((defined(FREETYPE_MAJOR)) && (((FREETYPE_MAJOR == 2) && (((FREETYPE_MINOR == 1) && (FREETYPE_PATCH >= 3)) || (FREETYPE_MINOR > 1))) || (FREETYPE_MAJOR > 2)))
       if (charmap->encoding == FT_ENCODING_MS_SYMBOL
-      ||  charmap->encoding == FT_ENCODING_ADOBE_CUSTOM
-      ||  charmap->encoding == FT_ENCODING_ADOBE_STANDARD) {
-      	a->have_char_map_unicode = 1;
-	found = charmap;
-	a->face->charmap = charmap;
-	return (void *)a;
-      }
+	  || charmap->encoding == FT_ENCODING_ADOBE_CUSTOM
+	  || charmap->encoding == FT_ENCODING_ADOBE_STANDARD)
+	{
+	  a->have_char_map_unicode = 1;
+	  found = charmap;
+	  a->face->charmap = charmap;
+	  return (void *) a;
+	}
 #endif /* Freetype 2.1 or better */
 /* EAM DEBUG */
 
@@ -632,6 +699,7 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 {
   unsigned char *pixel = NULL;
   int *tpixel = NULL;
+  int opixel;
   int x, y, row, col, pc, pcr;
 
   tweencolor_t *tc_elem;
@@ -649,7 +717,7 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 	  pcr = pc;
 	  y = pen_y + row;
 	  /* clip if out of bounds */
-          /* 2.0.16: clipping rectangle, not image bounds */		
+	  /* 2.0.16: clipping rectangle, not image bounds */
 	  if ((y > im->cy2) || (y < im->cy1))
 	    continue;
 	  for (col = 0; col < bitmap.width; col++, pc++)
@@ -677,6 +745,9 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 		{
 		  return "Unsupported ft_pixel_mode";
 		}
+              if (level == 0)  /* if background */
+		continue;
+
 	      if ((fg >= 0) && (im->trueColor))
 		{
 		  /* Consider alpha in the foreground color itself to be an
@@ -687,10 +758,10 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 		    level * (gdAlphaMax -
 			     gdTrueColorGetAlpha (fg)) / gdAlphaMax;
 		}
-	      level = gdAlphaMax - level;
+	      level = gdAlphaMax - level;   /* inverting to get alpha */
 	      x = pen_x + col;
 	      /* clip if out of bounds */
-              /* 2.0.16: clip to clipping rectangle, Matt McNabb */
+	      /* 2.0.16: clip to clipping rectangle, Matt McNabb */
 	      if ((x > im->cx2) || (x < im->cx1))
 		continue;
 	      /* get pixel location in gd buffer */
@@ -706,9 +777,16 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 		{
 		  if (im->alphaBlendingFlag)
 		    {
-		      *tpixel =
-			gdAlphaBlend (*tpixel,
+		      opixel = *tpixel;
+		      if (gdTrueColorGetAlpha(opixel) != gdAlphaTransparent)
+			{
+			  *tpixel = gdAlphaBlend (opixel,
 				      (level << 24) + (fg & 0xFFFFFF));
+			}
+		      else
+			{
+			  *tpixel = (level << 24) + (fg & 0xFFFFFF);
+			}
 		    }
 		  else
 		    {
@@ -761,65 +839,55 @@ gdft_draw_bitmap (gdCache_head_t * tc_cache, gdImage * im, int fg,
 	    {
 	      return "Unsupported ft_pixel_mode";
 	    }
-	  if (tc_key.pixel > 0)	/* if not background */
+	  if (tc_key.pixel == 0)	/* if background */
+	    continue;
+
+	  x = pen_x + col;
+
+	  /* clip if out of bounds */
+	  if (x >= im->sx || x < 0)
+	    continue;
+	  /* get pixel location in gd buffer */
+	  pixel = &im->pixels[y][x];
+	  if (tc_key.pixel == NUMCOLORS)
 	    {
-	      x = pen_x + col;
+	      /* use fg color directly. gd 2.0.2: watch out for
+	         negative indexes (thanks to David Marwood). */
+	      *pixel = (fg < 0) ? -fg : fg;
+	    }
+	  else
+	    {
+	      /* find antialised color */
 
-	      /* clip if out of bounds */
-	      if (x >= im->sx || x < 0)
-		continue;
-	      /* get pixel location in gd buffer */
-	      pixel = &im->pixels[y][x];
-	      if (tc_key.pixel == NUMCOLORS)
-		{
-		  /* use fg color directly. gd 2.0.2: watch out for
-		     negative indexes (thanks to David Marwood). */
-		  *pixel = (fg < 0) ? -fg : fg;
-		}
-	      else
-		{
-		  /* find antialised color */
-
-		  tc_key.bgcolor = *pixel;
-		  tc_elem = (tweencolor_t *) gdCacheGet (tc_cache, &tc_key);
-		  *pixel = tc_elem->tweencolor;
-		}
+	      tc_key.bgcolor = *pixel;
+	      tc_elem = (tweencolor_t *) gdCacheGet (tc_cache, &tc_key);
+	      *pixel = tc_elem->tweencolor;
 	    }
 	}
     }
   return (char *) NULL;
 }
 
-static int
-gdroundupdown (FT_F26Dot6 v1, int updown)
-{
-  return (!updown)
-    ? (v1 < 0 ? ((v1 - 63) >> 6) : v1 >> 6)
-    : (v1 > 0 ? ((v1 + 63) >> 6) : v1 >> 6);
-}
-
 extern int any2eucjp (char *, char *, unsigned int);
 
 /* Persistent font cache until explicitly cleared */
 /* Fonts can be used across multiple images */
 
 /* 2.0.16: thread safety (the font cache is shared) */
-gdMutexDeclare(gdFontCacheMutex);
+gdMutexDeclare (gdFontCacheMutex);
 static gdCache_head_t *fontCache;
 static FT_Library library;
 
-void
-gdFreeFontCache ()
+BGD_DECLARE(void) gdFreeFontCache ()
 {
-  gdFontCacheShutdown();
+  gdFontCacheShutdown ();
 }
 
-void
-gdFontCacheShutdown ()
+BGD_DECLARE(void) gdFontCacheShutdown ()
 {
   if (fontCache)
     {
-      gdMutexShutdown(gdFontCacheMutex);
+      gdMutexShutdown (gdFontCacheMutex);
       gdCacheDelete (fontCache);
       FT_Done_FreeType (library);
       /* 2.0.16: Gustavo Scotti: make sure we don't free this twice */
@@ -830,39 +898,37 @@ gdFontCacheShutdown ()
 /********************************************************************/
 /* gdImageStringFT -  render a utf8 string onto a gd image          */
 
-char *
-gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
+BGD_DECLARE(char *) gdImageStringFT (gdImage * im, int *brect, int fg, char *fontlist,
 		 double ptsize, double angle, int x, int y, char *string)
 {
   return gdImageStringFTEx (im, brect, fg, fontlist,
 			    ptsize, angle, x, y, string, 0);
 }
 
-int gdFontCacheSetup(void)
+BGD_DECLARE(int) gdFontCacheSetup (void)
 {
-	if (fontCache) {
-		/* Already set up */
-		return 0;
-	}
-	gdMutexSetup(gdFontCacheMutex);
-	if (FT_Init_FreeType (&library))
-	{
-		gdMutexShutdown(gdFontCacheMutex);
-		return -1;
-	}
-	fontCache = gdCacheCreate (FONTCACHESIZE,
-		 fontTest, fontFetch, fontRelease);
-	return 0;
+  if (fontCache)
+    {
+      /* Already set up */
+      return 0;
+    }
+  gdMutexSetup (gdFontCacheMutex);
+  if (FT_Init_FreeType (&library))
+    {
+      gdMutexShutdown (gdFontCacheMutex);
+      return -1;
+    }
+  fontCache = gdCacheCreate (FONTCACHESIZE, fontTest, fontFetch, fontRelease);
+  return 0;
 }
 
-char *
-gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
+
+BGD_DECLARE(char *) gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
 		   double ptsize, double angle, int x, int y, char *string,
 		   gdFTStringExtraPtr strex)
 {
-  FT_BBox bbox, glyph_bbox;
   FT_Matrix matrix;
-  FT_Vector pen, delta, penf;
+  FT_Vector penf, delta, total_min, total_max, glyph_min, glyph_max;
   FT_Face face;
   FT_Glyph image;
   FT_GlyphSlot slot;
@@ -871,8 +937,7 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
   FT_UInt glyph_index, previous;
   double sin_a = sin (angle);
   double cos_a = cos (angle);
-  int len, i = 0, ch;
-  int x1 = 0, y1 = 0;
+  int len, i, ch;
   font_t *font;
   fontkey_t fontkey;
   char *next;
@@ -880,7 +945,7 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
   int render = (im && (im->trueColor || (fg <= 255 && fg >= -255)));
   FT_BitmapGlyph bm;
   /* 2.0.13: Bob Ostermann: don't force autohint, that's just for testing 
-    freetype and doesn't look as good */
+     freetype and doesn't look as good */
   int render_mode = FT_LOAD_DEFAULT;
   int m, mfound;
   /* Now tuneable thanks to Wez Furlong */
@@ -894,6 +959,8 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
    *   colorindexes.          -- 27.06.2001 <krisku@arrak.fi>
    */
   gdCache_head_t *tc_cache;
+  /* Tuneable horizontal and vertical resolution in dots per inch */
+  int hdpi, vdpi;
   if (strex)
     {
       if ((strex->flags & gdFTEX_LINESPACE) == gdFTEX_LINESPACE)
@@ -908,13 +975,14 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
 /***** initialize font library and font cache on first call ******/
   if (!fontCache)
     {
-      if (gdFontCacheSetup() != 0) {
-	gdCacheDelete (tc_cache);
-        return "Failure to initialize font library";
-      }
+      if (gdFontCacheSetup () != 0)
+	{
+	  gdCacheDelete (tc_cache);
+	  return "Failure to initialize font library";
+	}
     }
 /*****/
-  gdMutexLock(gdFontCacheMutex);
+  gdMutexLock (gdFontCacheMutex);
   /* get the font (via font cache) */
   fontkey.fontlist = fontlist;
   fontkey.library = &library;
@@ -922,17 +990,29 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
   if (!font)
     {
       gdCacheDelete (tc_cache);
-      gdMutexUnlock(gdFontCacheMutex);
+      gdMutexUnlock (gdFontCacheMutex);
       return fontCache->error;
     }
   face = font->face;		/* shortcut */
   slot = face->glyph;		/* shortcut */
 
-  if (FT_Set_Char_Size (face, 0, (FT_F26Dot6) (ptsize * 64),
-			GD_RESOLUTION, GD_RESOLUTION))
+  /*
+   * Added hdpi and vdpi to support images at non-screen resolutions, i.e. 300 dpi TIFF,
+   *    or 100h x 50v dpi FAX format. 2.0.23.
+   * 2004/02/27 Mark Shackelford, mark.shackelford@acs-inc.com
+   */
+  hdpi = GD_RESOLUTION;
+  vdpi = GD_RESOLUTION;
+  if (strex && (strex->flags & gdFTEX_RESOLUTION))
+    {
+      hdpi = strex->hdpi;
+      vdpi = strex->vdpi;
+    }
+
+  if (FT_Set_Char_Size (face, 0, (FT_F26Dot6) (ptsize * 64), hdpi, vdpi))
     {
       gdCacheDelete (tc_cache);
-      gdMutexUnlock(gdFontCacheMutex);
+      gdMutexUnlock (gdFontCacheMutex);
       return "Could not set character size";
     }
 
@@ -941,10 +1021,6 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
   matrix.xy = -matrix.yx;
   matrix.yy = matrix.xx;
 
-  penf.x = penf.y = 0;		/* running position of non-rotated string */
-  pen.x = pen.y = 0;		/* running position of rotated string */
-  bbox.xMin = bbox.xMax = bbox.yMin = bbox.yMax = 0;
-
   use_kerning = FT_HAS_KERNING (face);
   previous = 0;
   if (fg < 0)
@@ -993,7 +1069,7 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
   if (!mfound)
     {
       /* No character set found! */
-      gdMutexUnlock(gdFontCacheMutex);
+      gdMutexUnlock (gdFontCacheMutex);
       return "No character set found";
     }
 
@@ -1017,17 +1093,18 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
       next = string;
     }
 #endif
-  while (*next)
+
+  penf.x = penf.y = 0;	/* running position of non-rotated glyphs */
+
+  for (i=0; *next; i++)
     {
+
       ch = *next;
 
       /* carriage returns */
       if (ch == '\r')
 	{
 	  penf.x = 0;
-	  x1 = (penf.x * cos_a - penf.y * sin_a + 32) / 64;
-	  y1 = (penf.x * sin_a + penf.y * cos_a + 32) / 64;
-	  pen.x = pen.y = 0;
 	  previous = 0;		/* clear kerning flag */
 	  next++;
 	  continue;
@@ -1035,95 +1112,96 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
       /* newlines */
       if (ch == '\n')
 	{
-          /* 2.0.13: reset penf.x. Christopher J. Grayce */
+	  /* 2.0.13: reset penf.x. Christopher J. Grayce */
 	  penf.x = 0;
-	  penf.y -= face->size->metrics.height * linespace;
-	  penf.y = (penf.y - 32) & -64;	/* round to next pixel row */
-	  x1 = (penf.x * cos_a - penf.y * sin_a + 32) / 64;
-	  y1 = (penf.x * sin_a + penf.y * cos_a + 32) / 64;
-	  pen.x = pen.y = 0;
+	  penf.y += face->size->metrics.height * linespace;
+	  penf.y = (penf.y + 32) & -64;	/* round to next pixel row */
 	  previous = 0;		/* clear kerning flag */
 	  next++;
 	  continue;
 	}
 
 /* EAM DEBUG */
-#if (defined(FREETYPE_MAJOR) && (FREETYPE_MAJOR >=2 ) && (FREETYPE_MINOR >= 1))
+/* TBB: get this exactly right: 2.1.3 *or better*, all possible cases. */
+/* 2.0.24: David R. Morrison: use the more complete ifdef here. */
+
+#if ((defined(FREETYPE_MAJOR)) && (((FREETYPE_MAJOR == 2) && (((FREETYPE_MINOR == 1) && (FREETYPE_PATCH >= 3)) || (FREETYPE_MINOR > 1))) || (FREETYPE_MAJOR > 2)))
       if (font->face->charmap->encoding == FT_ENCODING_MS_SYMBOL)
 	{
-         /* I do not know the significance of the constant 0xf000. */
-         /* It was determined by inspection of the character codes */
-         /* stored in Microsoft font symbol.ttf                    */
-           len = gdTcl_UtfToUniChar (next, &ch);
-           ch |= 0xf000;
-           next += len;
-	} else
+	  /* I do not know the significance of the constant 0xf000. */
+	  /* It was determined by inspection of the character codes */
+	  /* stored in Microsoft font symbol.ttf                    */
+	  len = gdTcl_UtfToUniChar (next, &ch);
+	  ch |= 0xf000;
+	  next += len;
+	}
+      else
 #endif /* Freetype 2.1 or better */
 /* EAM DEBUG */
-	
-      switch (m)
-	{
-	case gdFTEX_Unicode:
-	  if (font->have_char_map_unicode)
-	    {
-	      /* use UTF-8 mapping from ASCII */
-	      len = gdTcl_UtfToUniChar (next, &ch);
-	      next += len;
-	    }
-	  break;
-	case gdFTEX_Shift_JIS:
-	  if (font->have_char_map_sjis)
-	    {
-	      unsigned char c;
-	      int jiscode;
-	      c = *next;
-	      if (0xA1 <= c && c <= 0xFE)
-		{
-		  next++;
-		  jiscode = 0x100 * (c & 0x7F) + ((*next) & 0x7F);
 
-		  ch = (jiscode >> 8) & 0xFF;
-		  jiscode &= 0xFF;
-
-		  if (ch & 1)
-		    jiscode += 0x40 - 0x21;
-		  else
-		    jiscode += 0x9E - 0x21;
-
-		  if (jiscode >= 0x7F)
-		    jiscode++;
-		  ch = (ch - 0x21) / 2 + 0x81;
-		  if (ch >= 0xA0)
-		    ch += 0x40;
-
-		  ch = (ch << 8) + jiscode;
-		}
-	      else
-		{
-		  ch = c & 0xFF;	/* don't extend sign */
-		}
-	      next++;
-	    }
-	  break;
-	case gdFTEX_Big5:
+	switch (m)
 	  {
-	    /*
-	     * Big 5 mapping:
-	     * use "JIS-8 half-width katakana" coding from 8-bit characters. Ref:
-	     * ftp://ftp.ora.com/pub/examples/nutshell/ujip/doc/japan.inf-032092.sjs
-	     */
-	    ch = (*next) & 0xFF;	/* don't extend sign */
-	    next++;
-	    if (ch >= 161	/* first code of JIS-8 pair */
-		&& *next)
-	      {			/* don't advance past '\0' */
-		/* TBB: Fix from Kwok Wah On: & 255 needed */
-		ch = (ch * 256) + ((*next) & 255);
+	  case gdFTEX_Unicode:
+	    if (font->have_char_map_unicode)
+	      {
+		/* use UTF-8 mapping from ASCII */
+		len = gdTcl_UtfToUniChar (next, &ch);
+		next += len;
+	      }
+	    break;
+	  case gdFTEX_Shift_JIS:
+	    if (font->have_char_map_sjis)
+	      {
+		unsigned char c;
+		int jiscode;
+		c = *next;
+		if (0xA1 <= c && c <= 0xFE)
+		  {
+		    next++;
+		    jiscode = 0x100 * (c & 0x7F) + ((*next) & 0x7F);
+
+		    ch = (jiscode >> 8) & 0xFF;
+		    jiscode &= 0xFF;
+
+		    if (ch & 1)
+		      jiscode += 0x40 - 0x21;
+		    else
+		      jiscode += 0x9E - 0x21;
+
+		    if (jiscode >= 0x7F)
+		      jiscode++;
+		    ch = (ch - 0x21) / 2 + 0x81;
+		    if (ch >= 0xA0)
+		      ch += 0x40;
+
+		    ch = (ch << 8) + jiscode;
+		  }
+		else
+		  {
+		    ch = c & 0xFF;	/* don't extend sign */
+		  }
 		next++;
 	      }
+	    break;
+	  case gdFTEX_Big5:
+	    {
+	      /*
+	       * Big 5 mapping:
+	       * use "JIS-8 half-width katakana" coding from 8-bit characters. Ref:
+	       * ftp://ftp.ora.com/pub/examples/nutshell/ujip/doc/japan.inf-032092.sjs
+	       */
+	      ch = (*next) & 0xFF;	/* don't extend sign */
+	      next++;
+	      if (ch >= 161	/* first code of JIS-8 pair */
+		  && *next)
+		{		/* don't advance past '\0' */
+		  /* TBB: Fix from Kwok Wah On: & 255 needed */
+		  ch = (ch * 256) + ((*next) & 255);
+		  next++;
+		}
+	    }
+	    break;
 	  }
-	  break;
-	}
       /* set rotation transform */
       FT_Set_Transform (face, &matrix, NULL);
       /* Convert character code to glyph index */
@@ -1134,7 +1212,6 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
 	{
 	  FT_Get_Kerning (face, previous, glyph_index,
 			  ft_kerning_default, &delta);
-	  pen.x += delta.x;
 	  /* 2.0.12: indispensable for an accurate bounding box. */
 	  penf.x += delta.x;
 	}
@@ -1144,51 +1221,56 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
       if (err)
 	{
 	  gdCacheDelete (tc_cache);
-          gdMutexUnlock(gdFontCacheMutex);
+	  gdMutexUnlock (gdFontCacheMutex);
 	  return "Problem loading glyph";
 	}
 
-      /* transform glyph image */
-      FT_Get_Glyph (slot, &image);
       if (brect)
 	{			/* only if need brect */
-	  FT_Glyph_Get_CBox (image, ft_glyph_bbox_gridfit, &glyph_bbox);
-	  glyph_bbox.xMin += penf.x;
-	  glyph_bbox.yMin += penf.y;
-	  glyph_bbox.xMax += penf.x;
-	  glyph_bbox.yMax += penf.y;
-	  if (ch == ' ')	/* special case for trailing space */
-	    glyph_bbox.xMax += slot->metrics.horiAdvance;
-	  if (!i)
-	    {			/* if first character, init BB corner values */
-	      bbox.xMin = glyph_bbox.xMin;
-	      bbox.yMin = glyph_bbox.yMin;
-	      bbox.xMax = glyph_bbox.xMax;
-	      bbox.yMax = glyph_bbox.yMax;
+
+	  glyph_min.x = penf.x + slot->metrics.horiBearingX;
+	  glyph_min.y = penf.y - slot->metrics.horiBearingY;
+
+	  if (ch == ' ')        /* special case for trailing space */
+            {
+	      glyph_max.x = penf.x + slot->metrics.horiAdvance;
+            }
+          else
+            {
+	      glyph_max.x = glyph_min.x + slot->metrics.width;
+            }
+	  glyph_max.y = glyph_min.y + slot->metrics.height;
+
+	  if (i==0)
+	    {
+	      total_min = glyph_min;
+	      total_max = glyph_max;
 	    }
 	  else
 	    {
-	      if (bbox.xMin > glyph_bbox.xMin)
-		bbox.xMin = glyph_bbox.xMin;
-	      if (bbox.yMin > glyph_bbox.yMin)
-		bbox.yMin = glyph_bbox.yMin;
-	      if (bbox.xMax < glyph_bbox.xMax)
-		bbox.xMax = glyph_bbox.xMax;
-	      if (bbox.yMax < glyph_bbox.yMax)
-		bbox.yMax = glyph_bbox.yMax;
+              if (glyph_min.x < total_min.x)
+                  total_min.x = glyph_min.x;
+              if (glyph_min.y < total_min.y)
+                  total_min.y = glyph_min.y;
+              if (glyph_max.x > total_max.x)
+                  total_max.x = glyph_max.x;
+              if (glyph_max.y > total_max.y)
+                  total_max.y = glyph_max.y;
 	    }
-	  i++;
 	}
 
       if (render)
 	{
+          /* transform glyph image */
+          FT_Get_Glyph (slot, &image);
+
 	  if (image->format != ft_glyph_format_bitmap)
 	    {
 	      err = FT_Glyph_To_Bitmap (&image, ft_render_mode_normal, 0, 1);
 	      if (err)
 		{
 		  gdCacheDelete (tc_cache);
-                  gdMutexUnlock(gdFontCacheMutex);
+		  gdMutexUnlock (gdFontCacheMutex);
 		  return "Problem rendering glyph";
 		}
 	    }
@@ -1196,53 +1278,43 @@ gdImageStringFTEx (gdImage * im, int *brect, int fg, char *fontlist,
 	  /* now, draw to our target surface */
 	  bm = (FT_BitmapGlyph) image;
 	  gdft_draw_bitmap (tc_cache, im, fg, bm->bitmap,
-			    x + x1 + ((pen.x + 31) >> 6) + bm->left,
-			    y - y1 + ((pen.y + 31) >> 6) - bm->top);
+		    x + (penf.x * cos_a + penf.y * sin_a + 32)/64 + bm->left,
+		    y - (penf.x * sin_a - penf.y * cos_a - 32)/64 - bm->top);
+
+          FT_Done_Glyph (image);
 	}
 
       /* record current glyph index for kerning */
       previous = glyph_index;
 
-      /* increment pen position */
-      pen.x += image->advance.x >> 10;
-      pen.y -= image->advance.y >> 10;
-
+      /* advance character position */ 
       penf.x += slot->metrics.horiAdvance;
-
-      FT_Done_Glyph (image);
     }
 
   if (brect)
     {				/* only if need brect */
-      /* For perfect rounding, must get sin(a + pi/4) and sin(a - pi/4). */
-      double d1 = sin (angle + 0.78539816339744830962);
-      double d2 = sin (angle - 0.78539816339744830962);
 
-      /* rotate bounding rectangle */
-      brect[0] = (int) (bbox.xMin * cos_a - bbox.yMin * sin_a);
-      brect[1] = (int) (bbox.xMin * sin_a + bbox.yMin * cos_a);
-      brect[2] = (int) (bbox.xMax * cos_a - bbox.yMin * sin_a);
-      brect[3] = (int) (bbox.xMax * sin_a + bbox.yMin * cos_a);
-      brect[4] = (int) (bbox.xMax * cos_a - bbox.yMax * sin_a);
-      brect[5] = (int) (bbox.xMax * sin_a + bbox.yMax * cos_a);
-      brect[6] = (int) (bbox.xMin * cos_a - bbox.yMax * sin_a);
-      brect[7] = (int) (bbox.xMin * sin_a + bbox.yMax * cos_a);
-
-      /* scale, round and offset brect */
-      brect[0] = x + gdroundupdown (brect[0], d2 > 0);
-      brect[1] = y - gdroundupdown (brect[1], d1 < 0);
-      brect[2] = x + gdroundupdown (brect[2], d1 > 0);
-      brect[3] = y - gdroundupdown (brect[3], d2 > 0);
-      brect[4] = x + gdroundupdown (brect[4], d2 < 0);
-      brect[5] = y - gdroundupdown (brect[5], d1 > 0);
-      brect[6] = x + gdroundupdown (brect[6], d1 < 0);
-      brect[7] = y - gdroundupdown (brect[7], d2 < 0);
+      /* increase by 1 pixel to allow for rounding */
+      total_min.x -= 64;
+      total_min.y -= 64;
+      total_max.x += 64;
+      total_max.y += 64;
+ 
+      /* rotate bounding rectangle, scale and round to int pixels, and translate */
+      brect[0] = x + (total_min.x * cos_a + total_max.y * sin_a + 32)/64;
+      brect[1] = y - (total_min.x * sin_a - total_max.y * cos_a + 32)/64;
+      brect[2] = x + (total_max.x * cos_a + total_max.y * sin_a + 32)/64;
+      brect[3] = y - (total_max.x * sin_a - total_max.y * cos_a + 32)/64;
+      brect[4] = x + (total_max.x * cos_a + total_min.y * sin_a + 32)/64;
+      brect[5] = y - (total_max.x * sin_a - total_min.y * cos_a + 32)/64;
+      brect[6] = x + (total_min.x * cos_a + total_min.y * sin_a + 32)/64;
+      brect[7] = y - (total_min.x * sin_a - total_min.y * cos_a + 32)/64;
     }
 
   if (tmpstr)
     gdFree (tmpstr);
   gdCacheDelete (tc_cache);
-  gdMutexUnlock(gdFontCacheMutex);
+  gdMutexUnlock (gdFontCacheMutex);
   return (char *) NULL;
 }
 
